@@ -57,6 +57,21 @@ DefaultEncoding = utf-8
 ; Show the bozo flag for each badly formed feed
 ShowBozo = 0
 
+; Reload feeds automatically
+AutoUpdate = 0
+
+; Auto-reload interval (no effect unless AutoUpdate is used)
+UpdateInterval = 300
+
+; Expire old feed items on refresh
+Expire = 1
+
+; Number of minutes to keep old articles (defaults to 10 days)
+; Note: overriden by ExpireAmount
+ExpireTime = 864000
+
+; Keep only the x last number of feed items (overrides ExpireTime)
+ExpireAmount = 0
 
 [Colors]
 
@@ -101,9 +116,9 @@ class Config:
                      self._get('General', 'DefaultConfigFile'))
 
         for loc in locations:
-            print loc
+            #print loc
             if os.path.exists(loc):
-                print "exists", loc
+                #print "exists", loc
                 self._set('General', 'ConfigFile', loc)
                 return loc
 
@@ -114,7 +129,7 @@ class Config:
 
 
     def _get(self, section, option):
-        return self.config.get(section, option).lower()
+        return self.config.get(section, option)#.lower()
 
         
     def _set(self, section, option,value):
@@ -122,7 +137,7 @@ class Config:
 
     def write(self):
         tempfd, tempfname = tempfile.mkstemp(prefix='goodnews-')
-        print tempfname
+        #print tempfname
         fp = os.fdopen(tempfd, 'w')
         self.config.remove_section('System')
         self.config.write(fp)
@@ -215,3 +230,34 @@ class Config:
         else:
             self._set('General', 'ShowBozo', '0')
             
+    def get_autoupdate(self):
+        return self._get('General', 'AutoUpdate')
+
+    def set_autoupdate(self, flag):
+        val = '1' if flag else '0'
+        self._set('General', 'AutoUpdate', val)
+
+    def get_updateinterval(self):
+        return self._get('General', 'UpdateInterval')
+
+    def set_autoupdate(self, interval):
+        interval = str(interval)
+        self._set('General', 'UpdateInterval', interval)
+
+    def get_expire(self):
+        return bool(int(self._get('General', 'Expire')))
+
+    def set_expire(self, flag):
+        self._set('General', 'Expire', str(int(flag)))
+
+    def get_expiretime(self):
+        return int(self._get('General', 'ExpireTime'))
+
+    def set_expiretime(self, t):
+        self._set('General', 'ExpireTime', str(int(t)))
+
+    def get_expireamount(self):
+        return int(self._get('General', 'ExpireAmount'))
+
+    def set_expireamout(self, i):
+        self._set('General', 'ExpireAmount', str(int(i)))
