@@ -13,6 +13,8 @@ import sha
 import textwrap
 import cPickle
 import threading
+import math
+import md5
 
 #import ctypescurses._ctypescurses as curses
 
@@ -26,6 +28,7 @@ import feedparser
 import html2text
 
 
+
 # use_default_colors was added in python 2.4
 # if we don't have it, don't bother with colors
 if not hasattr(curses, "use_default_colors"):
@@ -56,7 +59,7 @@ class GoodnewsURLopener(urllib.FancyURLopener):
     version = USER_AGENT
 urllib._urlopener = GoodnewsURLopener()
 
-
+update_all_running = 0
 CMD_SLOTS_COLUMNS =6 
 CMD_SLOTS = (CMD_SLOTS_COLUMNS * 2) - 1
 
@@ -66,6 +69,76 @@ keybinding = {}
 first_ptr = []
 first_scr_ptr = 0
 
+
+
+class Pager:
+
+    def __init__(self, rows, cols, txt=""):
+        self.rows, self.cols = rows, cols
+        self.pages = 0
+        self.page  = 0
+        self.offset = 0
+        self.lines = []
+        if txt:
+            self._data = self.data(txt)
+
+    def resize(self, rows, cols):
+        self.rows, self.cols = rows, cols
+        self.data(self._data)
+        #self.home()
+
+    def data(self, txt=""):
+        if txt:
+            self._data = txt
+            lines = txt.split('\n')
+            self.lines = [line[:self.cols] for line in lines]
+            self.pages = int(math.ceil(len(self.lines) / (self.rows)))
+            self.home()
+        else:
+            return "\n".join(self.lines[(self.rows * self.page) + self._rel_offset(): (self.rows * (self.page+1)) + self._rel_offset()])
+            
+
+    def up(self, npages=1):
+        if self.page: 
+            self.page -= 1
+            self.offset = self._cur_offset(self.page)
+            return 1
+        else:
+            self.offset = 0
+            return 0
+
+    def down(self, npages=1):
+        if self.page < self.pages: 
+            self.page += 1
+            self.offset = self._cur_offset(self.page)
+            return 1
+        else:
+            return 0
+
+    def home(self):
+        self.page = self.offset = 0
+
+    def end(self):
+        self.page = self.pages
+        self.offset = 0
+
+    def scroll(self, nlines=1):
+        if self.offset < (len(self.lines) - self.rows):
+            self.offset += 1
+            self.page = self._cur_page(self.offset)
+
+    def _cur_page(self, offset):
+        self.page = int(math.ceil(offset / (self.rows)))
+        return self.page
+
+    def _rel_offset(self):
+        return self.offset - (self.page * self.rows)
+
+    def _cur_offset(self, page):
+        self.offset = (page * self.rows)
+        return self.offset
+
+
 class FeedMenu:
 
     def __init__(self):
@@ -78,7 +151,7 @@ class Feed:
     def __init__(self):
         self.id = ""
         self.feedurl = ""
-        self.feed = ""
+        #self.feed = ""
         self.feed_data = ""
         self.title = ""
         self.link = ""
@@ -89,7 +162,8 @@ class Feed:
         
         self.items = {}
         self.sorted = []
-        
+        self.expired = {}
+
         self.first_scr_ptr = 0
         
         self.problem = 0
@@ -99,6 +173,9 @@ class Feed:
         self.categories = ()
 
         self.bozo = 0
+        self.update_msg = ""
+
+        
 
 class NewsItem:
     def __init__(self):
@@ -106,11 +183,11 @@ class NewsItem:
         self.title = ""
         self.link = ""
         self.description = ""
+        self.html = ""
         self.id = ""
+        self.timestamp = 0
+        self.pager = None
 
-        self.page_lines = []
-        self.page_showed = 0
-        self.total_pages = 0
 
 
 def ex_entry_field(stdscr, default=""):
@@ -217,7 +294,6 @@ def draw_footer(stdscr, cur_grp, bindings):
     slots = CMD_SLOTS_COLUMNS
             
     grp = cur_grp[:]
-
     
     slotspace = columns // slots
 
@@ -271,7 +347,43 @@ def dump_traceback(stdscr, s):
 
 
 ## KEYBINDINGS ##
+def keylocked(fn):
+    def new(*args, **kws):
+        global has_keylock
+        keylock.acquire()
+        has_keylock = True
 
+        if not update_all_running:        
+            res = fn(*args, **kws)
+        else:
+            res = None
+
+        has_keylock = False
+        keylock.release()
+        return res
+    return new
+
+
+def do_unlocked(fn):
+    def new(*args, **kws):
+        global has_keylock
+        keylock_before = has_keylock
+
+        if keylock_before:
+            has_keylock = False
+            keylock.release()
+        
+        res = fn(*args, **kws)
+
+        if keylock_before:
+            keylock.acquire()
+            has_keylock = True
+
+        return res
+    return new
+
+
+@keylocked
 def keybind_morecmds(stdscr, **args):
     global cur_cmd_grp
 
@@ -304,13 +416,16 @@ def keybind_morecmds(stdscr, **args):
     draw_footer(stdscr, cur_cmd_grp, bindings)
 
 
+@keylocked
 def keybind_bozo(stdscr, **args):
     cfg.set_bozo(not cfg.get_bozo())
-    
+
+@keylocked    
 def keybind_quit(scr, **args):
     return 1
 
 
+@keylocked
 def keybind_addfeed(stdscr, **args):
     global highlighted
     
@@ -318,7 +433,8 @@ def keybind_addfeed(stdscr, **args):
     if ptr:
         highlighted = first_ptr[-1]
     
-        
+
+@keylocked        
 def keybind_delfeed(stdscr, **args):
 
     deletion = highlighted
@@ -331,12 +447,14 @@ def keybind_delfeed(stdscr, **args):
     first_ptr.remove(deletion)
     stdscr.clear()
 
+@keylocked
 def keybind_allread(stdscr, **args):
     for i in highlighted.items:
         highlighted.items[i].readstatus = 1
     pass
 
 
+@keylocked
 def keybind_browser(stdscr, **args):
     draw_status(stdscr, 'Enter a browser command. Example: lynx "%s" (%s is the URL)', 0)
     s = ex_entry_field(stdscr, cfg.get_browser())
@@ -346,6 +464,7 @@ def keybind_browser(stdscr, **args):
 
     draw_status(stdscr, '', 0)        
 
+@keylocked
 def keybind_editfeedurl(stdscr, **args):
     draw_status(stdscr, 'Edit the feed url', 0)    
     _url = highlighted.feedurl
@@ -355,7 +474,8 @@ def keybind_editfeedurl(stdscr, **args):
         highlighted.feedurl = url
         update_feed(stdscr, highlighted)
     draw_status(stdscr, '', 0)
-    
+
+@keylocked    
 def keybind_feedname(stdscr, **args):
     
     draw_status(stdscr, 'Enter a feed name', 0)
@@ -367,7 +487,8 @@ def keybind_feedname(stdscr, **args):
         highlighted.title = highlighted._title
 
     draw_status(stdscr, '', 0)
-        
+
+@keylocked        
 def keybind_moveup(stdscr, **args):
 
     i =  first_ptr.index(highlighted)
@@ -378,6 +499,7 @@ def keybind_moveup(stdscr, **args):
 
     first_ptr.insert(i, highlighted)
 
+@keylocked
 def keybind_movedown(stdscr, **args):
     i =  first_ptr.index(highlighted)
     cut = first_ptr.remove(highlighted)
@@ -387,20 +509,24 @@ def keybind_movedown(stdscr, **args):
 
     first_ptr.insert(i, highlighted)
 
+@keylocked
 def keybind_feedinfo(stdscr, **args):
     pass
 
+@keylocked
 def keybind_reload(stdscr, **args):
     update_feed(stdscr, highlighted)
 
+@keylocked
 def keybind_reloadall(stdscr, **args):
     update_all_feeds(stdscr, first_ptr, draw_feeds, highlighted, args['bindings'])
 
-
+@keylocked
 def keybind_saveall(stdscr, **args):
     draw_status(stdscr, "Saving all states...", 0)
     save_all(stdscr)
-    
+
+@keylocked    
 def keybind_urljump (stdscr, **args):
     LINES, COLS = stdscr.getmaxyx()
 
@@ -413,37 +539,50 @@ def keybind_urljump (stdscr, **args):
     if ret:
         draw_status(stdscr, "Error '%i' from browser command..."% ret, 2)
 
+@keylocked
 def keybind_urljump2(stdscr, **args):
     pass
 
+@keylocked
 def keybind_sortfeeds (stdscr, **args):
     pass
 
+"""
+@keylocked
 def keybind_pup(stdscr, **args):
     pass
 
+@keylocked
 def keybind_pdown(stdscr, **args):
     pass
+"""
 
+@keylocked
 def keybind_categorize(stdscr, **args):
     pass
 
+@keylocked
 def keybind_filter(stdscr, **args):
     pass
 
+@keylocked
 def keybind_filtercurrent(stdscr, **args):
     pass
 
+@keylocked
 def keybind_nofilter(stdscr, **args):
     pass
 
+@keylocked
 def keybind_help(stdscr, **args):
     pass
 
+@keylocked
 def keybind_about(stdscr, **args):
     pass
 
 
+@keylocked
 def keybind_next(stdscr, **args):
     global highlighted, first_scr_ptr
 
@@ -460,6 +599,7 @@ def keybind_next(stdscr, **args):
         first_scr_ptr += 1
 
 
+@keylocked
 def keybind_prev(stdscr, **args):
     global highlighted, first_scr_ptr
     
@@ -473,17 +613,23 @@ def keybind_prev(stdscr, **args):
         first_scr_ptr -= 1
     
 
+@keylocked
 def keybind_feed_down(stdscr, **args):
     LINES,COLS = stdscr.getmaxyx()
     maxlines = LINES - 6
-    
-    if cur_feed.sorted.index(cur_feed.highlighted.id) < len(cur_feed.items) -1 :
-        cur_feed.highlighted = cur_feed.items[cur_feed.sorted[cur_feed.sorted.index(cur_feed.highlighted.id) + 1]]
+    index = cur_feed.sorted.index(cur_feed.highlighted.id)
 
-    if (cur_feed.sorted.index(cur_feed.highlighted.id) - cur_feed.first_scr_ptr) > (maxlines - 4):
+    if index < len(cur_feed.items) -1 :
+        index += 1
+        cur_feed.highlighted = cur_feed.items[cur_feed.sorted[index]]
+
+    if (index - cur_feed.first_scr_ptr) > (maxlines - 4):
         cur_feed.first_scr_ptr += 1
-        
 
+    if cur_feed.highlighted.pager:
+        cur_feed.highlighted.pager.home()
+
+@keylocked
 def keybind_feed_up(stdscr, **args):
     if cur_feed.sorted.index(cur_feed.highlighted.id) > 0:
         cur_feed.highlighted = cur_feed.items[cur_feed.sorted[cur_feed.sorted.index(cur_feed.highlighted.id) - 1]]
@@ -491,17 +637,26 @@ def keybind_feed_up(stdscr, **args):
     if (cur_feed.sorted.index(cur_feed.highlighted.id) - cur_feed.first_scr_ptr) < 0:
         cur_feed.first_scr_ptr -= 1
     
+    if cur_feed.highlighted.pager:
+        cur_feed.highlighted.pager.home()
+
+@keylocked    
 def keybind_mark_item_read(stdscr, **args):
     cur_feed.highlighted.readstatus = 1
-    keybind_feed_down(stdscr)
+    do_unlocked(keybind_feed_down)(stdscr)
 
+
+@keylocked
 def keybind_mark_item_unread(stdscr, **args):
     cur_feed.highlighted.readstatus = 0
-    keybind_feed_down(stdscr)    
+    do_unlocked(keybind_feed_down)(stdscr)
 
-    
+
+@keylocked    
 def keybind_select(stdscr, **args):
-    global cur_cmd_grp
+    global cur_cmd_grp, mainmenu
+
+    mainmenu = False
     
     item_keybind = {}
 
@@ -525,7 +680,8 @@ def keybind_select(stdscr, **args):
     item_keybind[' '] = 'keybind_feed_select'    
 
     item_keybind['B'] = 'keybind_browser'
-    
+    item_keybind['V'] = 'debug_shell'
+
     if not first_ptr:
         return
 
@@ -561,7 +717,8 @@ def keybind_select(stdscr, **args):
             stdscr.clear()
             break
         
-
+    mainmenu = True
+        
 def setup_cmd_grp(bindings):
     
     cmd_grp = []
@@ -582,31 +739,83 @@ def setup_cmd_grp(bindings):
     return grp
 
 
+@keylocked
 def keybind_feed_enter_quit(stdscr, **args):
+
+    if not cur_feed.highlighted:
+        return 1
+    
+    # quit return if there are no more read items
+    if not [i for i in cur_feed.items.values() if not i.readstatus]:
+        curses.ungetch('q')
+        return 1
+
+
+    # find next unread
+
+    #index = cur_feed.sorted.index(cur_feed.highlighted.id)
+    """
+    for i in cur_feed.sorted[index:]:
+        cur_feed.first_scr_ptr += 1
+        if not cur_feed.items[i].readstatus:
+            cur_feed.highlighted = cur_feed.items[i]
+            #LINES,COLS = stdscr.getmaxyx()
+            #maxlines = LINES - 6
+            #if (cur_feed.sorted.index(cur_feed.highlighted.id) - cur_feed.first_scr_ptr) > (maxlines - 4):
+            #
+            #    cur_feed.first_scr_ptr = maxlines - 4
+
+            return 1
+    """
+
+    # if next unread
+
     while cur_feed.highlighted.id != cur_feed.sorted[-1]:
-        keybind_feed_down(stdscr)
+        do_unlocked(keybind_feed_down)(stdscr)
         if not cur_feed.highlighted.readstatus:
             return 1
+
+
+    # send quit to parent and quit
     curses.ungetch('q')
     return 1
 
+def keybind_item_pgdown(stdscr, **args):
+    if not cur_feed.highlighted.pager.down():
+        return keybind_feed_enter_quit(stdscr, **args)
+
+@keylocked
+def keybind_item_pgup(stdscr, **args):
+    cur_feed.highlighted.pager.up()
+
+@keylocked
 def keybind_feed_select(stdscr, **args):
-    global cur_cmd_grp, cur_keybind
+    global cur_cmd_grp, cur_keybind, mainmenu
+
+    mainmenu = False
     
     item_keybind = {}
     
     item_keybind['q'] = 'keybind_quit'
     item_keybind['o'] = 'keybind_urljump'
-    item_keybind[curses.KEY_UP] = 'keybind_feed_up'
     item_keybind['p'] = 'keybind_feed_up'
-    item_keybind[curses.KEY_DOWN] = 'keybind_feed_down'
     item_keybind['n'] = 'keybind_feed_down'
-
     item_keybind['\n'] = 'keybind_feed_enter_quit'
-    item_keybind[' '] = 'keybind_feed_enter_quit'
+    item_keybind[' '] = 'keybind_item_pgdown'
+    item_keybind['b'] = 'keybind_item_pgup'
     item_keybind['B'] = 'keybind_browser'
-    
+    item_keybind[curses.KEY_UP] = 'keybind_feed_up'
+    item_keybind[curses.KEY_DOWN] = 'keybind_feed_down'
+    item_keybind[curses.KEY_NPAGE] = 'keybind_item_pgdown'
+    item_keybind[curses.KEY_PPAGE] = 'keybind_item_pgup'
+    item_keybind['V'] = 'debug_shell'
     cur_cmd_grp = setup_cmd_grp(item_keybind)
+
+    # init the pager
+    LINES, COLS = stdscr.getmaxyx()
+    columns = COLS - 10
+    maxlines = LINES - 13
+
     stdscr.clear()    
     while 1:
         draw_item(stdscr, cur_feed)
@@ -617,13 +826,8 @@ def keybind_feed_select(stdscr, **args):
             break
 
 
-def do_item_loop(stdscr, item, keybindings):
-    while 1:
-        stdscr.getch()
-
-
 def enter_main(stdscr, ptr, keybindings, encoding='latin-1'):
-    global highlighted, first_ptr, saved_keybindings, syscall, cur_cmd_grp
+    global highlighted, first_ptr, saved_keybindings, syscall, cur_cmd_grp, mainmenu
 
     syscall = ""
 
@@ -691,89 +895,88 @@ def pretty_print_item(stdscr, description):
         stdscr.addstr("   " + line + '\n')
 
 
+def pretty_item_lines(item, encoding='latin-1'):
+    LINES, COLS = stdscr.getmaxyx()
+    columns = item.pager.cols
+    SNIP = item.pager.rows
 
-def pretty_item_lines(description):
+    if item.html:
+        html = item.html#.encode(encoding, 'replace')
+        html2text.BODY_WIDTH = item.pager.cols - 4
+        lines = html2text.html2text(html).splitlines()
+        outlines = []
+        for line in lines:
+            if line == '* * *':
+                line = "-"*columns
+            outlines.append("   " + line)        
+        return "\n".join(outlines).encode(encoding, 'replace')
+        
+    description = item.description.encode(encoding, 'replace')
 
     tp = []
     for n in description.split("\n"):
         tp.append(n.strip())
+    
 
     description = "\n".join(tp)
     paragraphs = description.split("\n\n")
 
     np = []
     for p in paragraphs:
-        np.append(textwrap.fill(p, columns))
+        np.append(textwrap.fill(p, columns - 4))
 
-    outtext = "\n".join(np)
+    outtext = "\n\n".join(np)
     outlines_raw = outtext.split('\n')
     outlines = []
     
     for line in outlines_raw:
         if line == '* * *':
             line = "-"*columns
-        outlines.append(line)
+        outlines.append("   " + line)
 
-    return outlines
+    return "\n".join(outlines)
 
   
-
-def test_pretty_print_item(description):
-    LINES, COLS = (50, 72)
-    columns = COLS - 10
-    SNIP = LINES - 13
-
-    tp = []
-    for n in description.split("\n"):
-        tp.append(n.strip())
-
-    description = "\n".join(tp)
-    paragraphs = description.split("\n\n")
-
-    np = []
-    for p in paragraphs:
-        np.append(textwrap.fill(p, columns) + '\n')
-
-    outtext = "\n".join(np)
-    outlines = outtext.split('\n')
-
-    if len(outlines) > SNIP:
-        outlines[SNIP - 1] = outlines[SNIP -1][:-3] + '...'
-    
-    for line in outlines[:SNIP]:
-        if line == '* * *':
-            line = "-"*columns
-            
-        sys.stdout.write("   " + line + '\n')
-   
-        
 def draw_item(stdscr, feed, use_colors=1, encoding='latin-1'):
     ypos = 4
     LINES, COLS = stdscr.getmaxyx()
     columns = COLS - 10
-    maxlines = LINES - 6
+    maxlines = LINES - 13
     
     item = feed.highlighted
+    
+    if not item.pager:
+        item.pager = Pager(maxlines, columns)
+        item.pager.data(pretty_item_lines(item))
+
+    if columns != item.pager.cols or maxlines != item.pager.rows:
+        item.pager.resize(maxlines, columns)
 
     stdscr.move(0,0)
     stdscr.clear()
 
     draw_header(stdscr, "")
 
-    if feed.description:
-        des = feed.description.encode(encoding, 'replace')
+    des = feed.description.encode(encoding, 'replace')
+    if len(des) > columns:
+        des = des[:columns - 5 - 3] + '...'
 
-        if len(des) > columns:
-            des = des[:columns - 5 - 3] + '...'
+    stdscr.addstr(2, 1, des, curses.A_BOLD)
 
-        stdscr.addstr(2, 1, des, curses.A_BOLD)
-
-    stdscr.move(5,4)
     stdscr.move(5,0)
-    pretty_print_item(stdscr, item.title.encode(encoding, 'replace'))
-    stdscr.addstr('\n   --\n')
 
-    pretty_print_item(stdscr, item.description.encode(encoding, 'replace'))
+    stdscr.attron(curses.A_BOLD)
+    pretty_print_item(stdscr, item.title.encode(encoding, 'replace'))
+    stdscr.attroff(curses.A_BOLD)
+
+    # draw page status
+    if item.pager.pages > 0:
+        status = "page: %s/%s" % (item.pager.page + 1, item.pager.pages +1)
+        stdscr.move(6, columns - len(status))
+        stdscr.addstr(status)
+    
+    stdscr.move(8,0)
+    stdscr.addstr(item.pager.data())
 
     draw_link(stdscr, item)
     
@@ -883,7 +1086,6 @@ def draw_feeds(stdscr, feeds, highlighted='', keybindings={}):
 
         center = columns // 2
         cos = len(outstr)// 2
-
         
         stdscr.addstr(10, center - cos, outstr)
         return
@@ -915,14 +1117,14 @@ def draw_feeds(stdscr, feeds, highlighted='', keybindings={}):
         else:
             stdscr.addstr(ypos, 1, title)
 
-        if newcount:
+        if feed.update_msg:
+            outstr = feed.update_msg
+        elif newcount:
             outstr = "(%i new)" % newcount
-            
         else:
             outstr = ""
-            
+
         stdscr.addstr(ypos, COLS - len(outstr) -1, outstr)            
-        
             
         if feed == highlighted:
             stdscr.attroff(curses.A_STANDOUT)
@@ -946,6 +1148,7 @@ def doupdatelink(stdscr, link):
             
 def do_main_cmdloop(stdscr, typeahead, keybindings):
     global cur_cmd_grp
+
     quit = 0
 
     uiinput = stdscr.getch();
@@ -959,42 +1162,24 @@ def do_main_cmdloop(stdscr, typeahead, keybindings):
             try:
                 uiinput = chr(uiinput)
             except ValueError:
-                #draw_status(stdscr, repr(repr(uiinput)) + "   ", 0)
                 pass
 
-                
         if keybindings.has_key(uiinput):
             method = keybindings[uiinput]
             g = globals()
 
             if g.has_key(method):
-                quit = g[method](stdscr, bindings=keybindings)
+                quit = do_unlocked(g[method])(stdscr, bindings=keybindings)
                 if quit:
                     return 1
         else:
-            #draw_status(stdscr, repr(repr(uiinput)) + "   ", 0)
             pass
 
 
-    if (uiinput == ord('\n')):
-	keybind_enter(stdscr)
-		
-    #/* TAB key is decimal 9. */
-    if (uiinput == chr(9) ):
-        pass
-
-    #/* ctrl+g clears typeahead. */
-    if (uiinput == chr(7) ):
-        pass
-
-    #/* ctrl+u clears line. */
-    if (uiinput == chr(21) ):
-        pass
-
     # ctrl+o shows next cmd group
     if (uiinput == chr(15) ):
-        keybind_morecmds(stdscr, bindings=keybindings)
-    
+        do_unlocked(keybind_morecmds)(stdscr, bindings=keybindings)
+
     return 0
 
 
@@ -1013,9 +1198,6 @@ class FeedParser:
 
         rssDocument = feedparser.parse(cur_ptr.feed_data)
 
-        if rssDocument.has_key('bozo') and rssDocument['bozo']:
-            cur_ptr.bozo = 1
-            
         try:
             # FIXME: when user has set title, don't check for updates,
             # otherwise check every time
@@ -1035,117 +1217,57 @@ class FeedParser:
         except:
             cur_ptr.description = ""
 
+        if not cur_ptr.description:
+            cur_ptr.description = cur_ptr.title
 
         for item in rssDocument['entries']:
 
-            if item.has_key('title'):
-                title = item['title']
-            else:
-                title = "(no title)"
-                
-            if item.has_key('link'):
-                link = item['link']
-                if link[0] == '/':
-                    _feed_url = urlparse.urlparse(cur_ptr.link)
-                    link = urllib.quote(link)
-                    link = "://".join(_feed_url[:2]) + link
-            else:
-                link = ""
+            new_item = NewsItem()
+            new_item.title = html2text.html2text(item.get('title') or "(no title)")
 
+            new_item.timestamp = int(time.time())
+
+            link = item.get('link')
+            if link[0] == '/':
+                _feed_url = urlparse.urlparse(cur_ptr.link)
+                link = urllib.quote(link)
+                link = "://".join(_feed_url[:2]) + link
+
+            new_item.link  = link
+
+            # find a unique id of sorts..
+            if item.has_key('guid'):
+                id = item['guid']
+            elif item.has_key('id'):
+                id = item['id']
+            else:
+                id = link
+
+            new_item.id = id
+
+            # get the description
             if item.has_key('description'):
-                description = item['description'] or "(no description)"
+                description = item['description']
             else:
                 description = "(no description)"
 
-            if item.has_key('id') and not (item['id'] == link):
-                id = item['id']
+            if has_html(description):
+                new_item.description = html2text.html2text(strip_links(description))
+                new_item.html = strip_links(description)
             else:
-                id = sha.new(link).hexdigest()
+                new_item.description = description
+            
+            if not cur_ptr.items.has_key(new_item.id):
+                cur_ptr.items[new_item.id] = new_item
+                orginal_sort.append(new_item.id)
 
-            if link:
-                new_item = NewsItem()
-                new_item.title = html2text.html2text(title)
-                new_item.link  = link
 
-                if has_html(description):
-                    #new_item.description = strip_tags(description)
-                    new_item.description = html2text.html2text(strip_links(description))
-                else:
-                    new_item.description = description
-                    
-                new_item.id = id
-            else:
-                return
+        cur_ptr.sorted = orginal_sort + cur_ptr.sorted
 
             
-            if item.has_key('readstatus'):
-                new_item.readstatus = int(item['readstatus'])
-            else:
-                new_item.readstatus = 0
 
-            
-            if new_item.link:
-                downloaded_items[new_item.id] = new_item
-                if new_item.id not in orginal_sort:
-                    orginal_sort.append(new_item.id)
-
-                    
-        # remove items no longer in print...
-        # FIXME: (add an expirery date function here)
-        for i in downloaded_items:
-            if cur_ptr.items.has_key(i):
-                downloaded_items[i].readstatus = cur_ptr.items[i].readstatus
-        cur_ptr.items = downloaded_items
-
-
-        # FIXME: add datetime to each item
-        cur_ptr.sorted = orginal_sort
-
-
-
-# to replace all those html &blabla; thingys
-def get_unicode_entitydefs():
-     import htmlentitydefs
-     entitydefs = htmlentitydefs.entitydefs.copy()
-     for k,v in entitydefs.items():
-         if v.startswith('&#'):
-             v = int(v[2:-1])
-         else:
-             v = ord(v)
-         entitydefs[k] = unichr(v)
-         
-     return entitydefs
-
-def replace_qoutes(txt):
-    import htmlentitydefs
-    qoutes = re.compile('&.*?;', re.S)
-    
-    ents = get_unicode_entitydefs()
-    all_qoutes = qoutes.findall(txt)
-    
-    for q in all_qoutes:
-        if q[2:-1].isdigit():
-            if int(q[2:-1]) == 8211:
-                new_chr = '\x2d'
-            if int(q[2:-1]) == 8212:
-                new_chr = ","
-            elif int(q[2:-1]) == 8216:
-                new_chr = "'"                
-            elif int(q[2:-1]) == 8217:
-                new_chr = "'"
-            else:
-                new_chr = unichr(int(q[2:-1]))
-                
-            txt = txt.replace(q, new_chr)
-            
-        elif ents.has_key(q[1:-1]):
-            c = ents[q[1:-1]]
-            txt = txt.replace(q, c)
-    return txt
-        
-
-# feeds are html, me don't want that :-)
 def strip_tags(txt):
+    """ Strip html tags """
     if txt:
         tags = re.compile('<.*?>', re.S)
         return tags.sub('', txt)
@@ -1154,8 +1276,10 @@ def strip_tags(txt):
 def has_html(txt):
     if not txt: return 0
     tags = re.compile('<.*?>', re.S)
-    if tags.search(txt):
+    qoutes = re.compile('&.*?;', re.S)
+    if tags.search(txt) or qoutes.search(txt):
         return 1
+
     
 def strip_links(txt):
 
@@ -1279,8 +1403,8 @@ def load_state_config(stdscr):
     #keybinding[curses.KEY_RIGHT] = 'keybind_next'
     keybinding[curses.KEY_UP] = 'keybind_prev'
     keybinding[curses.KEY_DOWN] = 'keybind_next'
-    keybinding[curses.KEY_NPAGE] = 'keybind_pdown'
-    keybinding[curses.KEY_PPAGE] = 'keybind_pup'
+    #keybinding[curses.KEY_NPAGE] = 'keybind_pdown'
+    #keybinding[curses.KEY_PPAGE] = 'keybind_pup'
     keybinding[curses.KEY_RESIZE] = 'keybind_resize'
     
     #keybinding['\t'] = 'keybind_tab'
@@ -1290,6 +1414,7 @@ def load_state_config(stdscr):
     # ctrl-u
     #keybinding['\x15'] = 'keybind_ctrlu'
     #keybinding['^O'] = 'keybind_morecmds'
+    keybinding['V'] = 'debug_shell'
     
 
     # FIXME: read keybindings from file here
@@ -1317,31 +1442,26 @@ class UpdateFeedThread(threading.Thread):
         self.tlock.acquire()            
         draw_status(stdscr, txt, delay)
         self.tlock.release()            
-        
+
     def run(self):
         try:
             self.real_run()
-
             draw_feeds(self.stdscr, self.first_ptr, self.highlighted)
             draw_footer(self.stdscr, cur_cmd_grp, keybindings)
             stdscr.refresh()
-
         except:
             pass
 
-        
     def real_run(self):
-
         if not self.cur_ptr:
             return 0
         LINES, COLS = stdscr.getmaxyx()
-
+        #self.cur_ptr.update_msg = "(Downloading)"
         self.draw_status_tlock(stdscr, "Downloading " + self.cur_ptr.feedurl[:COLS], 0)
-
         try:
-            self.cur_ptr.feed = urllib.urlopen(self.cur_ptr.feedurl)
-            self.cur_ptr.feed_data = self.cur_ptr.feed.read()
-            self.cur_ptr.feed = StringIO.StringIO(self.cur_ptr.feed_data)
+            feed = urllib.urlopen(self.cur_ptr.feedurl)
+            feed_data = feed.read()
+            #feed = StringIO.StringIO(feed_data)
 
         except IOError, err:
             self.draw_status_tlock(stdscr, "Downloading: "+err[1][1], 1)
@@ -1350,23 +1470,29 @@ class UpdateFeedThread(threading.Thread):
             self.draw_status_tlock(stdscr,str(err), 1)
             return 1
 
+        self.cur_ptr.feed_data = feed_data
+        #self.cur_ptr.feed = feed
+
         if not self.cur_ptr.title:
             self.cur_ptr.title = self.cur_ptr.feedurl
         if not self.cur_ptr.link:
             self.cur_ptr.link = self.cur_ptr.feedurl
 
-        if not self.cur_ptr.feed:
-            return 0
-
-        self.draw_status_tlock(stdscr, "Parsing " + self.cur_ptr.feedurl[:COLS], 0)
+        #self.cur_ptr.update_msg = "(Parsing)"
+        self.draw_status_tlock(stdscr, "Parsing: " + self.cur_ptr.feedurl[:COLS], 0)
         xml = FeedParser(self.cur_ptr)
-        if self.cur_ptr.bozo:
-            self.draw_status_tlock(stdscr, "Warning: bozo XML! Maybe not a RSS feed?", 2)
 
+        #self.cur_ptr.update_msg = "(Expire)"
+        self.draw_status_tlock(stdscr, "Expire items: " + self.cur_ptr.feedurl[:COLS], 0)
+
+        expire_items(stdscr, self.cur_ptr)
 
         #Don't mess up highlight
         self.cur_ptr.highlighted = None
 
+        self.cur_ptr.update_msg = ""
+        self.draw_status_tlock(stdscr, "", 0)
+
         return 1
 
         
@@ -1381,19 +1507,21 @@ def update_feed(stdscr, cur_ptr):
     draw_status(stdscr, "Downloading " + cur_ptr.feedurl[:COLS], 0)
 
     try:
-        cur_ptr.feed = urllib.urlopen(cur_ptr.feedurl)
-        cur_ptr.feed_data = cur_ptr.feed.read()
-        cur_ptr.feed = StringIO.StringIO(cur_ptr.feed_data)
+        feed = urllib.urlopen(cur_ptr.feedurl)
+        feed_data = feed.read()
+        #cur_ptr.feed = StringIO.StringIO(cur_ptr.feed_data)
         
     except IOError, err:
-        draw_status(stdscr, "Downloading: "+err[1][1], 1)
+        draw_status(stdscr, "Download failed: "+err[1][1], 1)
         return 2
     except timeoutsocket.Timeout, err:
         draw_status(stdscr,str(err), 1)
         return 1
-
-    if not cur_ptr.feed:
-        return 0
+    
+    cur_ptr.feed_data = feed_data
+    
+    #if not cur_ptr.feed:
+    #    return 0
 
     draw_status(stdscr, "Parsing " + cur_ptr.feedurl[:COLS], 0)
     xml = FeedParser(cur_ptr)
@@ -1402,22 +1530,22 @@ def update_feed(stdscr, cur_ptr):
         draw_status(stdscr, "Warning: bozo XML! Maybe not a RSS feed?", 2)
 
     #Don't mess up highlight
-    cur_ptr.highlighted = None
-    draw_status(stdscr, "", 0)
+    #cur_ptr.highlighted = None
+    
+    expire_items(stdscr, cur_ptr)
+
+    #draw_status(stdscr, "", 0)
     return 1
 
 
 def update_all_feeds(stdscr, first_ptr, draw_feeds=None, highlighted='', keybindings={}):
-    for cur_ptr in tuple(first_ptr):
-        if update_feed(stdscr, cur_ptr) == 2:
-            break
-
-        if draw_feeds:
-            draw_feeds(stdscr, first_ptr, highlighted, keybindings)
-            stdscr.refresh()
-
-
-def update_all_feeds(stdscr, first_ptr, draw_feeds=None, highlighted='', keybindings={}):
+    global update_all_running 
+    
+    if updatelock.acquire(False) == False:
+        return
+    
+    update_all_running = 1
+    
     tlock = threading.Lock()
 
     threads = []
@@ -1425,6 +1553,7 @@ def update_all_feeds(stdscr, first_ptr, draw_feeds=None, highlighted='', keybind
     # before we begin, redraw the feeds without a highlight bar
     draw_feeds(stdscr, first_ptr, highlighted='')
 
+    draw_status(stdscr, "Updating all feeds", 0)
     for cur_ptr in tuple(first_ptr):
 
         threads.append(UpdateFeedThread(tlock, stdscr, cur_ptr, first_ptr, draw_feeds, highlighted, keybindings))
@@ -1432,22 +1561,66 @@ def update_all_feeds(stdscr, first_ptr, draw_feeds=None, highlighted='', keybind
         threads[-1].setDaemon(0)
         threads[-1].start()
 
-    # don't return until all threads are finished,
-    # or else we risk all kind of bad things :-) 
-    while threading.activeCount() > 1:
-        pass
+    # wait for the threads
+    for t in threads:
+        t.join()
 
-    draw_status(stdscr, "", 0)
+
+    #draw_status(stdscr, "", 0)
+    update_all_running = 0
+    updatelock.release()
+
+    draw_status(stdscr, "Update complete.", 0)
+    draw_footer(stdscr, cur_cmd_grp, keybindings)
+    stdscr.refresh()
+
+def expire_items(stdscr, cur_ptr):
+
+    if not cfg.get_expire(): 
+        return
+    
+    amount = cfg.get_expireamount()
+    timeout = cfg.get_expiretime() * 60 
+    now = int(time.time())
+
+    # remove items we have already expired
+    for item in cur_ptr.sorted[:]:
+        hash = md5.md5(cur_ptr.items[item].link).hexdigest()
+        if hash in cur_ptr.expired:
+            #cur_ptr.expired[hash] = cur_ptr.items[item].readstatus 
+            cur_ptr.sorted.remove(item)
+            del(cur_ptr.items[item])
+            
+    # amount overrides
+    if 0: #amount:
+        for item in cur_ptr.sorted[amount:]: 
+            cur_ptr.expired[hash] = cur_ptr.items[item].readstatus 
+            del(cur_ptr.items[item])
+        cur_ptr.sorted = cur_ptr.sorted[:amount]
+    else:
+        for item in cur_ptr.sorted:
+            if (cur_ptr.items[item].timestamp + timeout) < now:
+                i = cur_ptr.items[item]
+                hash = md5.md5(i.link).hexdigest()
+                cur_ptr.expired[hash] = i.readstatus 
+
+                del(cur_ptr.items[item])
+                cur_ptr.sorted.remove(item)
+
+    #draw_status(stdscr, "", 0)
+    cur_ptr.highlighted = None
 
 #/* Load feed from disk. And call update_feed if neccessary. */
 def load_feed(stdscr,url):
 
-    hashme = hashurl(url)
-    fname = cfg.get_configdir() + "/cache/%s.pickle" % (hashme)
+    fname = cfg.get_configdir() + "/cache/%s.pickle" % (hashurl(url))
 
     try:
         cache = open(fname)
         feed = cPickle.load(cache)
+
+        feed.update_msg = ""
+
         return feed
     except:
         tmp = "Cache for %s is toast. Reloading from server..." % url
@@ -1456,72 +1629,18 @@ def load_feed(stdscr,url):
         feed.feedurl = url        
         if update_feed(stdscr, feed):
             return feed
-            
 
-
-class LoadFeedThread(threading.Thread):
-
-    def __init__(self, tlock, stdscr, cur_ptr):
+class UpdateThread(threading.Thread):
+    def __init__(self, stdscr, interval=300):
         threading.Thread.__init__(self)
-
-        self.tlock = tlock
         self.stdscr = stdscr
-        self.cur_ptr = cur_ptr
-
-
-    def draw_status_tlock(self, stdscr, txt, delay=0):
-        self.tlock.acquire()            
-        draw_status(stdscr, txt, delay)
-        self.tlock.release()            
-        
+        self.sleeptime = interval
+  
     def run(self):
-        try:
-            self.real_run()
-        except:
-            pass
-
-    def real_run(self):
-
-        tmp = "Loading cache for %s..." % self.cur_ptr.feedurl
-        self.draw_status_tlock(stdscr, tmp, 0)        
-
-        hashme = hashurl(self.cur_ptr.feedurl)
-        fname = cfg.get_configdir() + "/cache/%s" % (hashme)
-
-        try:
-            cache = open(fname)
-        except IOError:
-            tmp = "Cache for %s is toast. Reloading from server..." % self.cur_ptr.feedurl
-            self.draw_status_tlock(self.stdscr, tmp, 0);
-            if not update_feed(self.stdscr, self.cur_ptr):
-                first_ptr.remove(self.cur_ptr)
-
-            return 0
-
-
-        #/* Read complete cachefile. */
-        self.cur_ptr.feed = cache
-        self.cur_ptr.feed_data = self.cur_ptr.feed.read()
-        self.cur_ptr.feed = StringIO.StringIO(self.cur_ptr.feed_data)    
-
-        #/* After loading FeedParserize the mess. */
-        xml = FeedParser(self.cur_ptr)
-        cache.close()
-
-        if self.cur_ptr.bozo:
-            self.draw_status_tlock(self.stdscr, "Warning: bozo XML! Maybe not a RSS feed?", 2)
-
-
-        # get readstatus
-        fname = cfg.get_configdir() + "/cache-readstatus/%s" % (hashme)
-        cache_readstatus = open(fname)
-        for i in cache_readstatus.readlines():
-            id, status = i.split('\t')
-            if self.cur_ptr.items.has_key(id):
-                self.cur_ptr.items[id].readstatus = int(status)
-
-
-        return 1
+        while 1:
+            if mainmenu:
+                keylocked(update_all_feeds)(stdscr, first_ptr, draw_feeds, '', {})
+            time.sleep(self.sleeptime)
 
 
 def load_all_feeds(stdscr):
@@ -1551,6 +1670,11 @@ def save_feed_cache(stdscr, feed):
     hashme = hashurl(feed.feedurl);
     fname = cfg.get_configdir() + "/cache/%s.pickle" % (hashme)        
     cache = open(fname, "wb")
+    
+    # dont save the pager
+    for item in feed.items: 
+        feed.items[item].pager = None
+
     cPickle.dump(feed, cache)
     cache.close()
 
@@ -1583,14 +1707,31 @@ def add_feed(stdscr, first_ptr, default=""):
 
 def run(argv):
     global stdscr
+    global updatelock
+    global mainmenu
+    global keylock
+    global has_keylock
     #FIXME: argv checking
-    
+
+    has_keylock = False
+    mainmenu = True
+
     stdscr = InitCurses();
+    updatelock = threading.Lock()
+    keylock = threading.Lock()
 
     try:
+        stdscr.refresh()
         # load configs and caches
         load_state_config(stdscr);
 
+        # load autoupdater
+        if cfg.get_autoupdate() != '0':
+            interval = int(cfg.get_updateinterval());
+
+            update_thread = UpdateThread(stdscr, interval)
+            update_thread.start()
+
         #load_all_feeds(stdscr);        
         enter_main(stdscr, first_ptr, keybinding)
 
@@ -1607,10 +1748,27 @@ def run(argv):
         traceback.print_exception(sys.last_type, sys.last_value, sys.last_traceback, file=s)
         
         dump_traceback(stdscr, s.getvalue())
-        
 
     clean_up_and_exit("","")
 
 
+def debug_shell(stdscr, **args):
+    from code import InteractiveConsole
+    curses.reset_shell_mode()
+    import __main__
+    locals = __main__.__dict__
+    sh = InteractiveConsole(locals)
+    sh.interact()
+    stdscr.clear()
+    curses.reset_prog_mode()
+
+
 if __name__ == '__main__':
+    if len(sys.argv) == 2 and sys.argv[1] == '--clear-cache':
+        cfg = config.Config()
+        cachedir = cfg.get_cachedir()
+        print cachedir
+        ret = os.system('rm -rf %s' % cachedir)
+        sys.exit(ret)
+
     run(sys.argv)
